/*
	Author: Will Amesbury
	Created: 2015-09-18
	Purpose: ICDLogic javascript code for the NDoc diagnosis pages.
	History:
	2019-04-05 RMW ND-8597 Added 13 new additional diagnosis fields
	2018-07-27 DJA ND-6974 Add IcdLogicMenu as a class to all menus created here
	2015-10-15 WRA Corrected issue where ampersands in the search term could cause WebLink errors. ND-691.
	2015-10-15 WRA Corrected issue where the CypherLink menu item would be enabled for customers without ICDLogic. ND-670.
*/

var findButtons = [{ findButtonId: "SRCHDX0",   descriptionId: "D90T",     codeId: "D93T"   },
				   { findButtonId: "SRCHDX0P1", descriptionId: "D7093T",   codeId: "D7094T" },
				   { findButtonId: "SRCHDX0P2", descriptionId: "D7105T",   codeId: "D7106T" },

				   { findButtonId: "SRCHDX1",   descriptionId: "D2355T",   codeId: "D95T"   },
				   { findButtonId: "SRCHDX1P1", descriptionId: "D7095T",   codeId: "D7096T" },
				   { findButtonId: "SRCHDX1P2", descriptionId: "D7107T",   codeId: "D7108T" },

				   { findButtonId: "SRCHDX2",   descriptionId: "D2356T",   codeId: "D97T"   },
				   { findButtonId: "SRCHDX2P1", descriptionId: "D7097T",   codeId: "D7098T" },
				   { findButtonId: "SRCHDX2P2", descriptionId: "D7109T",   codeId: "D7110T" },

				   { findButtonId: "SRCHDX3",   descriptionId: "D2357T",   codeId: "D99T"   },
				   { findButtonId: "SRCHDX3P1", descriptionId: "D7099T",   codeId: "D7100T" },
				   { findButtonId: "SRCHDX3P2", descriptionId: "D7111T",   codeId: "D7112T" },

				   { findButtonId: "SRCHDX4",   descriptionId: "D2358T",   codeId: "D101T"  },
				   { findButtonId: "SRCHDX4P1", descriptionId: "D7101T",   codeId: "D7102T" },
				   { findButtonId: "SRCHDX4P2", descriptionId: "D7113T",   codeId: "D7114T" },

				   { findButtonId: "SRCHDX5",   descriptionId: "D2359T",   codeId: "D103T"  },
				   { findButtonId: "SRCHDX5P1", descriptionId: "D7103T",   codeId: "D7104T" },
				   { findButtonId: "SRCHDX5P2", descriptionId: "D7115T",   codeId: "D7116T" },

				   // Additional Dx
				   { findButtonId: "SRCHDXO1",  descriptionId: "D100352T", codeId: "D100351T" },
				   { findButtonId: "SRCHDXO2",  descriptionId: "D100354T", codeId: "D100353T" },

				   { findButtonId: "SRCHDXO3",  descriptionId: "D100356T", codeId: "D100355T" },
				   { findButtonId: "SRCHDXO4",  descriptionId: "D100358T", codeId: "D100357T" },

				   { findButtonId: "SRCHDXO5",  descriptionId: "D100360T", codeId: "D100359T" },
				   { findButtonId: "SRCHDXO6",  descriptionId: "D100362T", codeId: "D100361T" },
				
				   { findButtonId: "SRCHDXO7",  descriptionId: "D108621T", codeId: "D108620T" },
				   { findButtonId: "SRCHDXO8",  descriptionId: "D108629T", codeId: "D108628T" },
				
				   { findButtonId: "SRCHDXO9",  descriptionId: "D108637T", codeId: "D108636T" },
				   { findButtonId: "SRCHDXO10",  descriptionId: "D108645T", codeId: "D108644T" },
				
				   { findButtonId: "SRCHDXO11",  descriptionId: "D108653T", codeId: "D108652T" },
				   { findButtonId: "SRCHDXO12",  descriptionId: "D108661T", codeId: "D108660T" },
				
				   { findButtonId: "SRCHDXO13",  descriptionId: "D108669T", codeId: "D108668T" },
				   { findButtonId: "SRCHDXO14",  descriptionId: "D108677T", codeId: "D108676T" },
				
				   { findButtonId: "SRCHDXO15",  descriptionId: "D108685T", codeId: "D108684T" },
				   { findButtonId: "SRCHDXO16",  descriptionId: "D108693T", codeId: "D108692T" },
				
				   { findButtonId: "SRCHDXO17",  descriptionId: "D108701T", codeId: "D108700T" },
				   { findButtonId: "SRCHDXO18",  descriptionId: "D108709T", codeId: "D108708T" },

				   { findButtonId: "SRCHDXO19",  descriptionId: "D108717T", codeId: "D108716T" }];

var fieldButtonRelation = [{ id: "D90T",     type: "description", buttonId: "SRCHDX0"	},
						   { id: "D93T",     type: "code", 		  buttonId: "SRCHDX0"   },
						   { id: "D7093T",   type: "description", buttonId: "SRCHDX0P1" },
						   { id: "D7094T",   type: "code", 		  buttonId: "SRCHDX0P1" },
						   { id: "D7105T",   type: "description", buttonId: "SRCHDX0P2" },
						   { id: "D7106T",   type: "code", 		  buttonId: "SRCHDX0P2" },

						   { id: "D2355T",   type: "description", buttonId: "SRCHDX1"	},
						   { id: "D95T",     type: "code", 		  buttonId: "SRCHDX1"	},
						   { id: "D7095T",   type: "description", buttonId: "SRCHDX1P1" },
						   { id: "D7096T",   type: "code", 		  buttonId: "SRCHDX1P1" },
						   { id: "D7107T",   type: "description", buttonId: "SRCHDX1P2" },
						   { id: "D7108T",   type: "code", 		  buttonId: "SRCHDX1P2" },

						   { id: "D2356T",   type: "description", buttonId: "SRCHDX2"	},
						   { id: "D97T",     type: "code", 		  buttonId: "SRCHDX2"   },
						   { id: "D7097T",   type: "description", buttonId: "SRCHDX2P1" },
						   { id: "D7098T",   type: "code", 		  buttonId: "SRCHDX2P1" },
						   { id: "D7109T",   type: "description", buttonId: "SRCHDX2P2" },
						   { id: "D7110T",   type: "code", 		  buttonId: "SRCHDX2P2" },
						   
						   { id: "D2357T",   type: "description", buttonId: "SRCHDX3"	},
						   { id: "D99T",     type: "code", 		  buttonId: "SRCHDX3"   },
						   { id: "D7099T",   type: "description", buttonId: "SRCHDX3P1" },
						   { id: "D7100T",   type: "code", 		  buttonId: "SRCHDX3P1" },
						   { id: "D7111T",   type: "description", buttonId: "SRCHDX3P2" },
						   { id: "D7112T",   type: "code", 		  buttonId: "SRCHDX3P2" },
						   
						   { id: "D2358T",   type: "description", buttonId: "SRCHDX4"	},
						   { id: "D101T",    type: "code", 		  buttonId: "SRCHDX4"	},
						   { id: "D7101T",   type: "description", buttonId: "SRCHDX4P1" },
						   { id: "D7102T",   type: "code", 		  buttonId: "SRCHDX4P1" },
						   { id: "D7113T",   type: "description", buttonId: "SRCHDX4P2" },
						   { id: "D7114T",   type: "code", 		  buttonId: "SRCHDX4P2" },
						   
						   { id: "D2359T",   type: "description", buttonId: "SRCHDX5"	},
						   { id: "D103T",    type: "code", 		  buttonId: "SRCHDX5"	},
						   { id: "D7103T",   type: "description", buttonId: "SRCHDX5P1" },
						   { id: "D7104T",   type: "code", 		  buttonId: "SRCHDX5P1" },
						   { id: "D7115T",   type: "description", buttonId: "SRCHDX5P2" },
						   { id: "D7116T",   type: "code", 		  buttonId: "SRCHDX5P2" },

						   // Additional Dx
						   { id: "D100352T", type: "description", buttonId: "SRCHDXO1"  },
						   { id: "D100351T", type: "code", 		  buttonId: "SRCHDXO1"  },
						   { id: "D100354T", type: "description", buttonId: "SRCHDXO2"  },
						   { id: "D100353T", type: "code", 		  buttonId: "SRCHDXO2"  },

						   { id: "D100356T", type: "description", buttonId: "SRCHDXO3"  },
						   { id: "D100355T", type: "code", 		  buttonId: "SRCHDXO3"  },
						   { id: "D100358T", type: "description", buttonId: "SRCHDXO4"  },
						   { id: "D100357T", type: "code", 		  buttonId: "SRCHDXO4"  },

						   { id: "D100360T", type: "description", buttonId: "SRCHDXO5"  },
						   { id: "D100359T", type: "code", 		  buttonId: "SRCHDXO5"  },
						   { id: "D100362T", type: "description", buttonId: "SRCHDXO6"  },
						   { id: "D100361T", type: "code", 		  buttonId: "SRCHDXO6"  },
						
						   { id: "D108621T", type: "description", buttonId: "SRCHDXO7"  },
						   { id: "D108620T", type: "code", 		  buttonId: "SRCHDXO7"  },
						   { id: "D108629T", type: "description", buttonId: "SRCHDXO8"  },
						   { id: "D108628T", type: "code", 		  buttonId: "SRCHDXO8"  },
						
						   { id: "D108637T", type: "description", buttonId: "SRCHDXO9"  },
						   { id: "D108636T", type: "code", 		  buttonId: "SRCHDXO9"  },
						   { id: "D108645T", type: "description", buttonId: "SRCHDXO10"  },
						   { id: "D108644T", type: "code", 		  buttonId: "SRCHDXO10"  },
						
						   { id: "D108653T", type: "description", buttonId: "SRCHDXO11"  },
						   { id: "D108652T", type: "code", 		  buttonId: "SRCHDXO11"  },
						   { id: "D108661T", type: "description", buttonId: "SRCHDXO12"  },
						   { id: "D108660T", type: "code", 		  buttonId: "SRCHDXO12"  },
						
						   { id: "D108669T", type: "description", buttonId: "SRCHDXO13"  },
						   { id: "D108668T", type: "code", 		  buttonId: "SRCHDXO13"  },
						   { id: "D108677T", type: "description", buttonId: "SRCHDXO14"  },
						   { id: "D108676T", type: "code", 		  buttonId: "SRCHDXO14"  },
						
						   { id: "D108685T", type: "description", buttonId: "SRCHDXO15"  },
						   { id: "D108684T", type: "code", 		  buttonId: "SRCHDXO15"  },
						   { id: "D108693T", type: "description", buttonId: "SRCHDXO16"  },
						   { id: "D108692T", type: "code", 		  buttonId: "SRCHDXO16"  },
						
						   { id: "D108701T", type: "description", buttonId: "SRCHDXO17"  },
						   { id: "D108700T", type: "code", 		  buttonId: "SRCHDXO17"  },
						   { id: "D108709T", type: "description", buttonId: "SRCHDXO18"  },
						   { id: "D108708T", type: "code", 		  buttonId: "SRCHDXO18"  },

						   { id: "D108717T", type: "description", buttonId: "SRCHDXO19"  },
						   { id: "D108716T", type: "code", 		  buttonId: "SRCHDXO19"  }];

 // The onset date, exacerbation date, severity, and terminal relationship elements associated with the primary and other diagnoses
var associatedFields = [];
associatedFields[93]  = { onset:  94, exacerbation: 101161, severity: 6001, terminalRelationship: 100470 };
associatedFields[95]  = { onset:  96, exacerbation: 101162, severity: 6002, terminalRelationship: 100471 };
associatedFields[97]  = { onset:  98, exacerbation: 101163, severity: 6003, terminalRelationship: 100472 };
associatedFields[99]  = { onset: 100, exacerbation: 101164, severity: 6004, terminalRelationship: 100473 };
associatedFields[101] = { onset: 102, exacerbation: 101165, severity: 6005, terminalRelationship: 100474 };
associatedFields[103] = { onset: 104, exacerbation: 101166, severity: 6006, terminalRelationship: 100475 };

var dxJson = "";
var validationResult = "";
var monographWindow  = null;
var cypherLinkWindow = null;


function launchIcdLogic(vars, widgetType) {
	var searchTerm = document.getElementById(vars.codeId).value;
	widgetType = 'monograph';
	if (!searchTerm) {
		searchTerm = document.getElementById(vars.descriptionId).value;
		widgetType = 'cypher';
	}
	var post = 'searchTerm=' + encodeURIComponent(searchTerm) + '&widgetType=' + encodeURIComponent(widgetType);
	if (cypherLinkWindow === null || cypherLinkWindow.closed) {
		cypherLinkWindow = openIcdLogicWindow(post, widgetType);
	} else {
		cypherLinkWindow.focus();
	}
	/*
	if (widgetType === "cypher") {
		if (cypherLinkWindow === null || cypherLinkWindow.closed) {
			cypherLinkWindow = openIcdLogicWindow(post, widgetType);
		} else {
			cypherLinkWindow.focus();
		}
	} else if (widgetType === "monograph") {
		if (monographWindow === null || monographWindow.closed) {
			monographWindow = openIcdLogicWindow(post, widgetType);
		} else {
			monographWindow.focus();
		}
	}
	*/
}

function getFindButtonForField(fieldId) {
	var index;
	for (index = 0; index < fieldButtonRelation.length; ++index) {
		if (fieldButtonRelation[index].id == fieldId) {
			return fieldButtonRelation[index];
		}
	}
}

function receiveDiagnoses(diagnoses) {
	require(["dojo/json"], function(JSON) {
		// Perform validation
		Ign = "";
		validationResults = "";
		IcdLogicToNdoc(JSON.stringify(diagnoses));
		if ((validationResults !== "") && (Ign == "1")) {
			// There was a problem, so show error to user.
			cypherLinkWindow.saveStatus(validationResults);
			return;
		}

		var dx, de, code, field;
		for (var iDx in diagnoses) {
			// Go through once to clear out all changed codes so we don't get duplicate dx errors
			dx	  = diagnoses[iDx];
			de 	  = dx.de;
			code  = dx.code;
			field = document.getElementById("D" + de + "T");

			if (field && field.value != code) {
				//console.log("Clearing "+ de);
				field.value = "";
				if (typeof field.fireEvent !== "undefined") {
					field.fireEvent("onchange");
				} else {
					field.dispatchEvent(new Event("change"));
				}
			}
		}

		for (iDx in diagnoses) {
			// Now file non-blank diagnoses that changed
			dx	  = diagnoses[iDx];
			de	  = dx.de;
			code  = dx.code;
			field = document.getElementById("D" + de + "T");
			
			save(de, code, "text");

			// Save any onset, exacerbation, etc. fields if the new de has them
			if (associatedFields[de]) {
				if (dx.onset) {
					save(associatedFields[de].onset, dx.onset, "text");
				}
				if (dx.exacerbation) {
					save(associatedFields[de].exacerbation, dx.exacerbation, "text");
				}
				if (dx.severity) {
					save(associatedFields[de].severity, dx.severity, "radio");
				}
				if (dx.terminalRelationship) {
					save(associatedFields[de].terminalRelationship, dx.terminalRelationship, "select");
				}
			}
		}
		// No problem, so close the window.
		cypherLinkWindow.close();
	});
}

function save(de, value, type) {
	var field;

	if (type === "text") {
		field = document.getElementById("D" + de + "T");
		if (field && field.value != value) {
			field.value = value;
			if (typeof field.fireEvent !== "undefined") {
				field.fireEvent("onchange");
			} else {
				field.dispatchEvent(new Event("change"));
			}
		}
	} else if (type === "radio") {
		field = document.getElementById("D" + de + "R" + value);
		if (field && field.checked === false) {
			field.checked = true;
			if (typeof field.fireEvent !== "undefined") {
				field.fireEvent("onclick");
			} else {
				field.dispatchEvent(new Event("click"));
			}
		}
	} else if (type === "select") {
		field = document.getElementById("D" + de + "S");
		if (field && field.value != value) {
			field.value = value;
			if (typeof field.fireEvent !== "undefined") {
				field.fireEvent("onchange");
			} else {
				field.dispatchEvent(new Event("change"));
			}
		}
	}
}

require(['dojo/ready'], function(ready) {
	ready(function() {
		require(["dojo/parser",
				 "dojo/dom",
				 "dojo/aspect",
				 "dojo/json",
				 "dijit/registry",
				 "dijit/form/ComboButton",
				 "dijit/Menu",
				 "dijit/MenuItem"], function (parser, dom, aspect, JSON, registry, ComboButton, Menu, MenuItem) {
			parser.parse();

			if (icdLogicOn) {
				// Run after wldUpdatePage to catch which fields were changed
				aspect.after(window, "wldUpdatePage", getChangedFields);

				// Reset buttons after diagnoses are reordered 
				aspect.after(window, "Rd1", SetButtonStates);
				aspect.after(window, "Ru2", SetButtonStates);
				aspect.after(window, "Rd2", SetButtonStates);
				aspect.after(window, "Ru3", SetButtonStates);
				aspect.after(window, "Rd3", SetButtonStates);
				aspect.after(window, "Ru4", SetButtonStates);
				aspect.after(window, "Rd4", SetButtonStates);
				aspect.after(window, "Ru5", SetButtonStates);
				aspect.after(window, "Rd5", SetButtonStates);
				aspect.after(window, "Ru6", SetButtonStates);
			}

			var index;
			for (index = 0; index < findButtons.length; ++index) {
				var findButton = findButtons[index];

				// Create menus for buttons that exist
				if (registry.byId(findButton.findButtonId)) {
					createMenu(findButton);
				}
			}

			// Set initial button states
			SetButtonStates();

			function SetButtonStates() {
				for (var iField in fieldButtonRelation) {
					var field = fieldButtonRelation[iField].id;

					toggleMenuItem(field);
				}
			}

			// Extract names for the fields that changed from the global variable "resp", set by wldUpdatePage
			function getChangedFields() {
				for (var i in resp) {
					var response = resp[i];

					// Extract the field ID
					if (resp[i].substring(0, 23) == "self.document.forms[0].") {
						response = response.replace("self.document.forms[0].", "");
						var name = response.slice(0, response.indexOf("."));
						
						// the ID is name + "T"
						var id = name + "T";
						toggleMenuItem(id);
					}
				}
			}			
			
			function createMenu(vars) {
				var menu = new Menu({ className: "IcdLogicMenu", style: "display: none; width: 50px;"});

				createMenuItems(menu, vars);
				menu.startup();

				var button = registry.byId(vars.findButtonId);
				button.set("dropDown", menu);
			}

			function createMenuItems(menu, vars) {
				var codeGenius = new MenuItem({
					id: vars.findButtonId + "CodeGenius",
					label: "CodeGenius&trade;",
					disabled: !icdLogicOn,
					onClick: function(){ launchIcdLogic(vars, 'codeGenius'); }
				});
				menu.addChild(codeGenius);
				
				/*
				var cypherLink = new MenuItem({
					id: vars.findButtonId + "CypherLink",
					label: "CypherLink&trade;",
					disabled: !icdLogicOn,
					onClick: function(){ launchIcdLogic(vars.descriptionId, 'cypher'); }
				});
				menu.addChild(cypherLink);

				if (monographOn) {
					var monograph = new MenuItem({
						id: vars.findButtonId + "Monograph",
						label: "Monograph&#8480;",
						disabled: !icdLogicOn || isFieldBlank(vars.codeId),
						onClick: function(){ launchIcdLogic(vars.codeId, 'monograph'); }
					});
					menu.addChild(monograph);
				}
				*/
			}

			function isFieldBlank(fieldId) {
				var field = dom.byId(fieldId);
				if (field && field.value) {
					return true;
				}
				return false;
			}
		});
	});
});

function toggleMenuItem(fieldId) {
	require(["dojo/dom", "dijit/registry"], function (dom, registry) {
		if (icdLogicOn) {
			var relationObject = getFindButtonForField(fieldId);
			if (relationObject) {
				var buttonId = relationObject.buttonId;
				var type = relationObject.type;
				var field = dom.byId(fieldId);
				var menuItem;

				if (type == "description") {
					// menuItem = registry.byId(buttonId + "CypherLink");
					return; // CypherLink should always be enabled
				} else if (type == "code") {
					menuItem = registry.byId(buttonId + "Monograph");
				}

				if (menuItem && field) {
					if (field.value !== "") {
						menuItem.set("disabled", false);
					} else {
						menuItem.set("disabled", true);
					}
				}
			}
		}
	});
}