/*	
	Author: wamesbury
	Created: 2018-01-24
	Purpose: 
	History:
	2020-10-08 WRA ND-10550 Added "Last month" preset
*/
define([
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"thornberry/report/filter/_FilterMixin",
	"dojo/date",
	"dojo/date/locale",
	"dojo/on",
	"dojo/_base/lang",
	"dojo/_base/array",
	"thornberry/serverRequest",
	"dijit/form/DateTextBox",
	"dijit/form/MultiSelect"
], function (declare, _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _FilterMixin, date, locale, on, lang, arrayUtils, serverRequest) {
	return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
		value: { start: null, end: null },
		_fiscalYearStart: null,

		// lockable: boolean
        //      If false, then this field is not locked when defined by a preset
        lockable: false,
		
		templateString: "<div data-dojo-attach-point='focusNode,containerNode'>" +
							"<input data-dojo-type='dijit/form/DateTextBox' data-dojo-attach-point='startDate' data-dojo-attach-event='onChange: _updateConstraints' />" +
							"<span> through </span>" +
							"<input data-dojo-type='dijit/form/DateTextBox' data-dojo-attach-point='endDate' />" +
							"<span>&nbsp;</span>" +
							"<input data-dojo-type='dijit/form/Select' data-dojo-attach-point='preset' data-dojo-attach-event='onChange: _selectPreset' />" +
						"</div>",

        getFilterValue: function() {
            // Summary: 
            //      This returns the value to be used by the corresponding column class's GenerateWHERE() method
			
			var startFormatted = null;
			var endFormatted = null;

			if (this.startDate.validate()) {
				var startDate = this.startDate.get("value");
				startFormatted = (startDate !== null ? locale.format(startDate, { datePattern: "MM/dd/yyyy", selector: "date" }) : null);
			}
			if (this.endDate.validate()) {
				var endDate = this.endDate.get("value");
				endFormatted = (endDate !== null ? locale.format(endDate, { datePattern: "MM/dd/yyyy", selector: "date" }) : null);
			}
			
			return { start: startFormatted, end: endFormatted };
        },

        getDisplayValue: function() {
            // Summary: 
            //      This returns the value to be used for display in the generated report

			var filterVal = this.getFilterValue();
            var startFormatted = filterVal.start;
            var endFormatted = filterVal.end;
			var displayVal = "";
			
			var preset = this.preset.get("value");
			var startDate = this.startDate.get("value");
			var endDate = this.endDate.get("value");

            if (preset !== null && preset !== '') {
				displayVal = 'within ' + this.preset.get('displayedValue');
            } else if (startDate !== null && endDate !== null) {
				displayVal = startFormatted + " through " + endFormatted;
            } else if (startDate == null && endDate !== null) {
				displayVal = "on or before " + endFormatted;
            } else if (startDate !== null && endDate == null) {
				displayVal = "on or after " + startFormatted;
            }

            return displayVal;
        },

        serialize: function() {
            // Summary: 
			//      This returns the value to be stored in ^UserSettings

			var value = "";
			var preset = this.preset.get("value");

			if (preset !== "") {
				value = preset;
			} else {
				var startDate = this.startDate.get("value");
				var endDate = this.endDate.get("value");
				var today = new Date();
				today.setHours(0);
				today.setMinutes(0);
				today.setSeconds(0);
				today.setMilliseconds(0);
				
				var startOffset = (startDate !== null ? date.difference(today, startDate, "day") : null);
				var endOffset   = (endDate !== null ? date.difference(today, endDate, "day") : null);

				value = startOffset + "," + endOffset;
			}
			return value;
        },

        deserialize: function(value) {
            // Summary: 
			//      This sets the filter's value from the serialized value
			
			if (arrayUtils.indexOf(["today", "-3days", "-7days", "-14days", "-30days", "-1year", "+3days", "+7days", "+14days", "+30days", "wtd", "mtd", "qtd", "ytd", "lm", "lq", "lytd", "fytd"], value) !== -1) {
				this.preset.set("value", value);
				this._selectPreset();
			} else {
				this.preset.set("value", "");

				var startOffset = null;
				var endOffset = null;

				if (value !== null) {
					startOffset = value.split(",")[0];
					endOffset = value.split(",")[1];

					startOffset = (startOffset === "" ? null : +startOffset);
					endOffset   = (endOffset   === "" ? null : +endOffset);
				}
				
				this.startDate._onChangeActive = false;
				this.startDate.set("value", (startOffset !== null ? date.add(new Date(), "day", startOffset) : null));
				this.startDate._onChangeActive = true;

				this.endDate._onChangeActive = false;
				this.endDate.set("value", (endOffset !== null ? date.add(new Date(), "day", endOffset) : null));
				this.endDate._onChangeActive = true;

				this.onChange();
				this._updateConstraints();
			}
		},

		buildRendering: function() {
			this.inherited(arguments);
			this.getFiscalYear();

			var options = [];
			options.push({ label: "(Select)",				value: "" });
			options.push({ label: "Today",					value: "today" });
			options.push({ label: "Past 3 days",			value: "-3days" });
			options.push({ label: "Past 7 days",			value: "-7days" });
			options.push({ label: "Past 14 days",			value: "-14days" });
			options.push({ label: "Past 30 days",			value: "-30days" });
			options.push({ label: "Past year",				value: "-1year" });
			options.push({ label: "Next 3 days",			value: "+3days" });
			options.push({ label: "Next 7 days",			value: "+7days" });
			options.push({ label: "Next 14 days",			value: "+14days" });
			options.push({ label: "Next 30 days",			value: "+30days" });
			options.push({ label: "Week to date",			value: "wtd" });
			options.push({ label: "Month to date",			value: "mtd" });
			options.push({ label: "Quarter to date",		value: "qtd" });
			options.push({ label: "Year to date",			value: "ytd" });
			options.push({ label: "Last month",				value: "lm" });
			options.push({ label: "Last quarter",			value: "lq" });
			options.push({ label: "Fiscal year to date",	value: "fytd" });
			options.push({ label: "Last year to date",		value: "lytd" });

			this.preset.addOption(options);
		},
		
		postCreate: function() {
			var MINDATE = new Date("12/31/1840");
			this.startDate.constraints.min = MINDATE;
			this.endDate.constraints.min = MINDATE;

			this.own(on(this.startDate, "blur", lang.hitch(this, function () { checkForT(this.startDate.id); })),
					 on(this.endDate,   "blur", lang.hitch(this, function () { checkForT(this.endDate.id); })));

			this.own(on(this.startDate, "change", lang.hitch(this, "onChange")),
					 on(this.endDate,	"change", lang.hitch(this, "onChange")));

			this.own(on(this.startDate, "change", lang.hitch(this, "_clearPreset")),
					 on(this.endDate,	"change", lang.hitch(this, "_clearPreset")));
		},

		onChange: function() {},
		focus: function() {},

		validate: function() {
			return this.startDate.validate() && this.endDate.validate();
		},

		_selectPreset: function() {
			var preset = this.preset.get("value");

			var startDate, endDate;
			var today = new Date();

			this.startDate._onChangeActive = false;
			this.endDate._onChangeActive = false;

			if (preset == "today") {
				this.startDate.set("value", today, false);
				this.endDate.set("value", today, false);
			} else if (preset == "+3days") {
				this.startDate.set("value", today, false);
				this.endDate.set("value", date.add(today, "day", +3), false);
			} else if (preset == "+7days") {
				this.startDate.set("value", today, false);
				this.endDate.set("value", date.add(today, "day", +7), false);
			} else if (preset == "+14days") {
				this.startDate.set("value", today, false);
				this.endDate.set("value", date.add(today, "day", +14), false);
			} else if (preset == "+30days") {
				this.startDate.set("value", today, false);
				this.endDate.set("value", date.add(today, "day", +30), false);
			} else if (preset == "-3days") {
				this.startDate.set("value", date.add(today, "day", -3), false);
				this.endDate.set("value", today, false);
			} else if (preset == "-7days") {
				this.startDate.set("value", date.add(today, "day", -7), false);
				this.endDate.set("value", today, false);
			} else if (preset == "-14days") {
				this.startDate.set("value", date.add(today, "day", -14), false);
				this.endDate.set("value", today, false);
			} else if (preset == "-30days") {
				this.startDate.set("value", date.add(today, "day", -30), false);
				this.endDate.set("value", today, false);
			} else if (preset == "-1year") {
				this.startDate.set("value", date.add(today, "year", -1), false);
				this.endDate.set("value", today, false);
			} else if (preset == "wtd") {
				var dayOfWeek = today.getDay();
				this.startDate.set("value", date.add(today, "day", -dayOfWeek), false);
				this.endDate.set("value", today, false);
			} else if (preset == "mtd") {
				startDate = new Date(today);
				startDate.setDate(1);
				this.startDate.set("value", startDate, false);
				this.endDate.set("value", today, false);
			} else if (preset == "qtd") {
				var currentMonth = today.getMonth();
				var monthOfQuarter = currentMonth%3;
				startDate = new Date(today);
				startDate.setDate(1);
				startDate.setMonth(currentMonth - monthOfQuarter);
				this.startDate.set("value", startDate, false);
				this.endDate.set("value", today, false);
			}  else if (preset == "ytd") {
				startDate = new Date(today);
				startDate.setMonth(0);
				startDate.setDate(1);
				this.startDate.set("value", startDate, false);
				this.endDate.set("value", today, false);
			} else if (preset == "lm") {
				startDate = date.add(today, "month", -1);
				startDate.setDate(1);
				endDate = new Date(today);
				endDate.setDate(1);
				endDate = date.add(endDate, "day", -1);
				this.startDate.set("value", startDate, false);
				this.endDate.set("value", endDate, false);
			} else if (preset == "lq") {
				var currentMonth = today.getMonth();
				var monthOfQuarter = currentMonth%3;
				startDate = new Date(today);
				startDate.setDate(1);
				startDate.setMonth(currentMonth - monthOfQuarter - 3);
				endDate = new Date();
				endDate.setDate(1);
				endDate.setMonth(currentMonth - monthOfQuarter);
				this.startDate.set("value", startDate, false);
				this.endDate.set("value", date.add(endDate, "day", -1), false);
			} else if (preset == "lytd") {
				startDate =  new Date(today);
				startDate.setMonth(0);
				startDate.setDate(1);
				this.startDate.set("value", date.add(startDate, "year", -1), false);
				this.endDate.set("value", date.add(today, "year", -1), false);
			} else if (preset == "fytd") {
				this.startDate.set("value", this._fiscalYearStart, false);
				this.endDate.set("value", today, false);
			}

			this.startDate._onChangeActive = true;
			this.endDate._onChangeActive = true;

			this.onChange();
			this._updateConstraints();
		},

		_clearPreset: function() {
			this.preset.set("value", "");
		},

		_updateConstraints: function() {
			this.endDate.set("constraints", { min: this.startDate.get("value") });
			this.endDate.validate();
		},

		_getValueAttr: function() {
			var value = this.getFilterValue();
			if (value.start === null && value.end === null) {
				value = null;
			}
			return value;
		},

		_setDisabledAttr: function (/*Boolean*/ value) {
			this.startDate.set("disabled", value);
			this.endDate.set("disabled", value);
			this.preset.set("disabled", value);
		},

		getFiscalYear: function() {
			serverRequest("User.AgencyDemographics", "StartofFiscalYear").then(lang.hitch(this, function (results) {
				this._fiscalYearStart = locale.parse(results, { datePattern: "MM/dd/yyyy", selector: "date" });
			}), function (error) {
				console.error(error);
			});
		}
    });
});