/*	
	Author: dallison
	Created: 2024-10-09
	Purpose: 
	History:
*/
define([
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"thornberry/report/filter/_FilterMixin",
	"dojo/date",
	"dojo/date/locale",
	"dojo/on",
	"dojo/_base/lang",
	"dojo/_base/array",
	"thornberry/serverRequest",
	"dijit/form/DateTextBox",
	"dijit/form/MultiSelect",
    "thornberry/report/filter/MultiSelect",
    "thornberry/report/filter/DateRange"
], function (declare, _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _FilterMixin, date, locale, on, lang, arrayUtils, serverRequest) {
	return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _FilterMixin], {
		value: { documentDates: null, dueDateStatuses: null, dueDateRange: null },

		// lockable: boolean
        //      If false, then this field is not locked when defined by a preset
        lockable: false,

        optionsValue: null,
		
		templateString: "<div data-dojo-attach-point='focusNode,containerNode'>" +
						"<table><tr>" + 
							"<td valign=\"top\">Track By<br><span data-dojo-type='thornberry/report/filter/MultiSelect' data-dojo-attach-point='filterDocumentDates'></span></td>" +
							"<td valign=\"top\">Date Status<br><span data-dojo-type='thornberry/report/filter/MultiSelect' data-dojo-attach-point='filterDueDateStatuses'></span></td>" +
						"</tr><tr style=\"height:5px\"> </tr><tr>" + 
							"<td valign=\"top\" colspan=\"2\">Due Date: <span data-dojo-type='thornberry/report/filter/DateRange' data-dojo-attach-point='filterDueDateRange'></span></td>" +
						"</tr></table>" + 
						"</div>",

        getFilterValue: function() {
            // Summary: 
            //      This returns the value to be used by the corresponding column class's GenerateWHERE() method
			
			return {
				documentDates: this.filterDocumentDates.getFilterValue(),
				dueDateStatuses: this.filterDueDateStatuses.getFilterValue(),
				dueDateRange: this.filterDueDateRange.getFilterValue()
			};
        },

        getDisplayValue: function() {
            // Summary: 
            //      This returns the value to be used for display in the generated report
			var documentDates = this.filterDocumentDates.getDisplayValue();
			var dueDateStatuses = this.filterDueDateStatuses.getDisplayValue();
			var dueDateRange = this.filterDueDateRange.getDisplayValue();
			
			var displayValue = '';
			if ((dueDateStatuses == null) || (dueDateStatuses == '')) {
				displayValue += 'ALL Due Statuses';
			} else {
				var dueDateStatusesArray = dueDateStatuses.split(', ');
				if (dueDateStatusesArray.length > 1) {
					dueDateStatusesArray[dueDateStatusesArray.length - 1] = 'and ' + dueDateStatusesArray[dueDateStatusesArray.length - 1];
					if (dueDateStatusesArray.length > 2) {
						dueDateStatuses = dueDateStatusesArray.join(', ');
					} else {
						dueDateStatuses = dueDateStatusesArray.join(' ');
					}
					displayValue += dueDateStatuses;
				} else {
					displayValue += dueDateStatusesArray[0];
				}
			}
			
			displayValue += ' for ';
			
			if ((documentDates == null) || (documentDates == '')) {
				displayValue += 'ALL Document Dates';
			} else {
				var documentDatesArray = documentDates.split(', ');
				if (documentDatesArray.length > 1) {
					documentDatesArray[documentDatesArray.length - 1] = 'and ' + documentDatesArray[documentDatesArray.length - 1];
					if (documentDatesArray.length > 2) {
						documentDates = documentDatesArray.join(', ');
					} else {
						documentDates = documentDatesArray.join(' ');
					}
					displayValue += documentDates;
				} else {
					displayValue += documentDatesArray[0];
				}
			}
			
			displayValue += ' ';
			
			displayValue += dueDateRange;

            return displayValue;
        },

        serialize: function() {
            // Summary: 
			//      This returns the value to be stored in ^UserSettings

			var documentDates = this.filterDocumentDates.serialize();
			var dueDateStatuses = this.filterDueDateStatuses.serialize();
			var dueDateRange = this.filterDueDateRange.serialize();

			return documentDates + ':' + dueDateStatuses + ':' + dueDateRange;
        },

        deserialize: function(value) {
            // Summary: 
			//      This sets the filter's value from the serialized value
			
			if (value != null) {
				var values = value.split(':');
				
				if (values[0]) {
					this.filterDocumentDates.deserialize(values[0]);
				}
				if (values[1]) {
					this.filterDueDateStatuses.deserialize(values[1]);
				}
				if (values[2]) {
					this.filterDueDateRange.deserialize(values[2]);
				}
			}
			
		},

		buildRendering: function() {
			this.inherited(arguments);
		},
		
		postCreate: function() {
			this.own(on(this.filterDocumentDates, "change", lang.hitch(this, "onChange")),
					 on(this.filterDueDateStatuses,	"change", lang.hitch(this, "onChange")),
					 on(this.filterDueDateRange,	"change", lang.hitch(this, "onChange")));
		},

        _setOptionsValueAttr: function(options) {
            var t = this;
			t.filterDocumentDates.set('optionsValue',options.documentDates);
			t.filterDueDateStatuses.set('optionsValue',options.dueDateStatuses);
        },

		onChange: function() {},
		focus: function() {},

		validate: function() {
			return this.filterDueDateRange.validate();
		},

		_getValueAttr: function() {
			return this.getFilterValue();
		},

		_setDisabledAttr: function (/*Boolean*/ value) {
			this.filterDocumentDates.set("disabled", value);
			this.filterDueDateStatuses.set("disabled", value);
			this.filterDueDateRange.set("disabled", value);
		}
    });
});